@@ -5,6 +5,8 @@ from ctypes import (
     c_int,
     c_float,
     c_char_p,
+    c_int32,
+    c_uint32,
     c_void_p,
     c_bool,
     POINTER,
@@ -105,6 +107,9 @@ LLAMA_FILE_MAGIC_UNVERSIONED = LLAMA_FILE_MAGIC_GGML
 LLAMA_SESSION_MAGIC = LLAMA_FILE_MAGIC_GGSN
 LLAMA_SESSION_VERSION = c_int(1)
 
+# #define LLAMA_DEFAULT_SEED           0xFFFFFFFF
+LLAMA_DEFAULT_SEED = c_int(0xFFFFFFFF)
+
 # struct llama_model;
 llama_model_p = c_void_p
 
@@ -153,18 +158,17 @@ llama_progress_callback = ctypes.CFUNCTYPE(None, c_float, c_void_p)
 
 
 # struct llama_context_params {
-#     int seed;                              // RNG seed, -1 for random
-#     int n_ctx;                             // text context
-#     int n_batch;                           // prompt processing batch size
-#     int n_gpu_layers;                      // number of layers to store in VRAM
-#     int main_gpu;                          // the GPU that is used for scratch and small tensors
+#     uint32_t seed;                         // RNG seed, -1 for random
+#     int32_t  n_ctx;                        // text context
+#     int32_t  n_batch;                      // prompt processing batch size
+#     int32_t  n_gpu_layers;                 // number of layers to store in VRAM
+#     int32_t  main_gpu;                     // the GPU that is used for scratch and small tensors
 #     float tensor_split[LLAMA_MAX_DEVICES]; // how to split layers across multiple GPUs
 #     // called with a progress value between 0 and 1, pass NULL to disable
 #     llama_progress_callback progress_callback;
 #     // context pointer passed to the progress callback
 #     void * progress_callback_user_data;
 
-
 #     // Keep the booleans together to avoid misalignment during copy-by-value.
 #     bool low_vram;   // if true, reduce VRAM usage at the cost of performance
 #     bool f16_kv;     // use fp16 for KV cache
@@ -176,11 +180,11 @@ llama_progress_callback = ctypes.CFUNCTYPE(None, c_float, c_void_p)
 # };
 class llama_context_params(Structure):
     _fields_ = [
-        ("seed", c_int),
-        ("n_ctx", c_int),
-        ("n_batch", c_int),
-        ("n_gpu_layers", c_int),
-        ("main_gpu", c_int),
+        ("seed", c_uint32),
+        ("n_ctx", c_int32),
+        ("n_batch", c_int32),
+        ("n_gpu_layers", c_int32),
+        ("main_gpu", c_int32),
         ("tensor_split", c_float * LLAMA_MAX_DEVICES.value),
         ("progress_callback", llama_progress_callback),
         ("progress_callback_user_data", c_void_p),
@@ -290,13 +294,14 @@ _lib.llama_mlock_supported.restype = c_bool
 
 # // TODO: not great API - very likely to change
 # // Initialize the llama + ggml backend
+# // If numa is true, use NUMA optimizations
 # // Call once at the start of the program
-# LLAMA_API void llama_init_backend();
-def llama_init_backend():
-    return _lib.llama_init_backend()
+# LLAMA_API void llama_init_backend(bool numa);
+def llama_init_backend(numa: c_bool):
+    return _lib.llama_init_backend(numa)
 
 
-_lib.llama_init_backend.argtypes = []
+_lib.llama_init_backend.argtypes = [c_bool]
 _lib.llama_init_backend.restype = None
 
 
@@ -452,7 +457,7 @@ _lib.llama_get_kv_cache_token_count.restype = c_int
 
 # Sets the current rng seed.
 # LLAMA_API void llama_set_rng_seed(struct llama_context * ctx, int seed);
-def llama_set_rng_seed(ctx: llama_context_p, seed: c_int):
+def llama_set_rng_seed(ctx: llama_context_p, seed: c_uint32):
     return _lib.llama_set_rng_seed(ctx, seed)
 
 
@@ -565,6 +570,27 @@ _lib.llama_eval.argtypes = [llama_context_p, llama_token_p, c_int, c_int, c_int]
 _lib.llama_eval.restype = c_int
 
 
+# // Same as llama_eval, but use float matrix input directly.
+# LLAMA_API int llama_eval_embd(
+#         struct llama_context * ctx,
+#                     const float * embd,
+#                             int   n_tokens,
+#                             int   n_past,
+#                             int   n_threads);
+def llama_eval_embd(
+    ctx: llama_context_p,
+    embd,  # type: Array[c_float]
+    n_tokens: c_int,
+    n_past: c_int,
+    n_threads: c_int,
+) -> int:
+    return _lib.llama_eval_embd(ctx, embd, n_tokens, n_past, n_threads)
+
+
+_lib.llama_eval_embd.argtypes = [llama_context_p, c_float_p, c_int, c_int, c_int]
+_lib.llama_eval_embd.restype = c_int
+
+
 # Convert the provided text into tokens.
 # The tokens pointer must be large enough to hold the resulting tokens.
 # Returns the number of tokens on success, no more than n_max_tokens
@@ -998,5 +1024,5 @@ _lib.llama_print_system_info.restype = c_char_p
 _llama_initialized = False
 
 if not _llama_initialized:
-    llama_init_backend()
+    llama_init_backend(c_bool(False))
     _llama_initialized = True
